# Copyright (c) 1993 Xerox Corporation.
# Use and copying of this software and preparation of derivative works based
# upon this software are permitted. Any distribution of this software or
# derivative works must comply with all applicable United States export
# control laws. This software is made available AS IS, and Xerox Corporation
# makes no warranty about the software, its performance or its conformity to
# any specification.

# changes/additions by Jason Rennie <jrennie@ai.mit.edu>

###############
### msg.tcl ###
###############

# this procedure filters a message using ifilter
proc Msg_Filter {} {
    global ftoc exmh mhProfile

    set msg_num [Ftoc_MsgNumber $ftoc(curLine)]

    Exmh_Status "ifile filter ..."

    if [catch {exec ifilter.mh < "$mhProfile(path)/$exmh(folder)/$msg_num"} err] {
	Exmh_Debug Inc_iFile: $err
    } else {
	Msg_Remove
	Flist_FindUnseen 1
    }
}

###############
### inc.tcl ###
###############

proc Inc_Custom {} {
    # Transfer from the mail spool file directly into folders
    global exmh mhProfile env inc
    # Use inc to copy stuff into a temp directory
    if ![file isdirectory $mhProfile(path)/MyIncTmp] {
      exec mkdir $mhProfile(path)/MyIncTmp
    }
    if {[catch {exec inc +MyIncTmp -silent} err]} {
      # Some versions of inc exit non-zero when they should not.
      Exmh_Debug Inc_Presort +MyIncTmp: $err
    }
    if [catch {set env(USER)} user] {
      if [catch {set env(LOGNAME)} user] {
          Exmh_Status "No USER or LOGNAME envar"
          set user ""
      }
    }
    if {[catch {glob $mhProfile(path)/MyIncTmp/*} files] == 0} {
      Exmh_Status "ifile inc ..."
      foreach file $files {
          if {[string compare [file tail $file] "++"] == 0} {
              if [catch {exec rm $file} err] {
                  Exmh_Debug Inc_iFile: $err
              }
              continue
          }
          if {$inc(presortFeedback)} {
              if [catch {exec grep "^Subject:" $file | head -1} \
                  subject] {
                  set subject ""
              }
          }

          # now use ifilter to (presumably) shift things to a folder
          if [catch {exec ifilter.mh < $file} err] {
              # Move it out of MyIncTmp in case it really did
              # get filed somewhere.  Certain file system errors
              # (can't stat .) lead to this behavior
              Exmh_Status "$file - $err - check MyIncErrors"
              if ![file isdirectory $mhProfile(path)/MyIncErrors] {
                  exec mkdir $mhProfile(path)/MyIncErrors
              }
              Mh_Refile MyIncTmp [file tail $file] MyIncErrors 
          } else {
              if [catch {exec rm $file} err] {
                  Exmh_Debug Inc_iFile: $err
              }
              if {$inc(presortFeedback)} {
                  if {[string length $subject] > 0} {
                      regsub "^Subject: *" $subject "" subject
                      Exmh_Status "[file tail $file]: $subject" red
                  } else {
                      Exmh_Status [file tail $file] red
                  }
              }
          }
      }
    }
    catch {exec rm -f $mhProfile(path)/MyIncTmp/$mhProfile(mh-sequences)}
    catch {exec rmdir $mhProfile(path)/MyIncTmp}

    Flist_FindUnseen          ;# Needed to set new message state.
    # This after breaks a potential deadlock:
    # UI Inc button is pressed - registers outstanding Inc operation
    # PresortFinish notes new messages in the current folder and wants
    # to rescan the folder to pick up the changes.
    # The scan notes the outstanding operation and waits for it to complete.
    # Deadlock
    after 1 {BgRPC Inc_CustomFinish}
}
proc Inc_CustomFinish {} {
    global exmh ftoc
    Mh_Folder $exmh(folder)   ;# Inc_Custom inc has changed this to MyIncTmp
    if {$ftoc(displayValid) && [Flist_NumUnseen $exmh(folder)] > 0} {
	Label_Folder $exmh(folder)
	Scan_Folder $exmh(folder) $ftoc(showNew)
    }
}
